@@ -1,32 +1,41 @@
 # coding: utf-8
 from __future__ import print_function, unicode_literals
 
-import platform
-import time
-import sys
 import os
+import platform
+import sys
+import time
+
+try:
+    raise Exception()
+
+    raise Exception()
+except:
+    TYPE_CHECKING = False
 
 PY2 = sys.version_info[0] == 2
 if PY2:
     sys.dont_write_bytecode = True
-    unicode = unicode
+    unicode = unicode  # noqa: F821  # pylint: disable=undefined-variable,self-assigning-variable
 else:
     unicode = str
 
-WINDOWS = False
-if platform.system() == "Windows":
-    WINDOWS = [int(x) for x in platform.version().split(".")]
+WINDOWS  = (
+    [int(x) for x in platform.version().split(".")]
+    if platform.system() == "Windows"
+    else False
+)
 
 VT100 = not WINDOWS or WINDOWS >= [10, 0, 14393]
 # introduced in anniversary update
 
-ANYWIN = WINDOWS or sys.platform in ["msys"]
+ANYWIN = WINDOWS or sys.platform in ["msys", "cygwin"]
 
 MACOS = platform.system() == "Darwin"
 
 
-def get_unixdir():
-    paths = [
+def get_unixdir()  :
+    paths    = [
         (os.environ.get, "XDG_CONFIG_HOME"),
         (os.path.expanduser, "~/.config"),
         (os.environ.get, "TMPDIR"),
@@ -43,7 +52,7 @@ def get_unixdir():
                     continue
 
                 p = os.path.normpath(p)
-                chk(p)
+                chk(p)  # type: ignore
                 p = os.path.join(p, "copyparty")
                 if not os.path.isdir(p):
                     os.mkdir(p)
@@ -56,7 +65,7 @@ def get_unixdir():
 
 
 class EnvParams(object):
-    def __init__(self):
+    def __init__(self)  :
         self.t0 = time.time()
         self.mod = os.path.dirname(os.path.realpath(__file__))
         if self.mod.endswith("__init__"):
