@@ -2,27 +2,30 @@
 from __future__ import print_function, unicode_literals
 
 import os
-from ..util import fsenc, fsdec, SYMTIME
+
+from ..util import SYMTIME, fsdec, fsenc
 from . import path
 
+try:
+    raise Exception()
+except:
+    pass
+
+_ = (path,)
 
 # grep -hRiE '(^|[^a-zA-Z_\.-])os\.' . | gsed -r 's/ /\n/g;s/\(/(\n/g' | grep -hRiE '(^|[^a-zA-Z_\.-])os\.' | sort | uniq -c
 # printf 'os\.(%s)' "$(grep ^def bos/__init__.py | gsed -r 's/^def //;s/\(.*//' | tr '\n' '|' | gsed -r 's/.$//')"
 
 
-def chmod(p, mode):
+def chmod(p , mode )  :
     return os.chmod(fsenc(p), mode)
 
 
-def listdir(p="."):
+def listdir(p  = ".")  :
     return [fsdec(x) for x in os.listdir(fsenc(p))]
 
 
-def lstat(p):
-    return os.lstat(fsenc(p))
-
-
-def makedirs(name, mode=0o755, exist_ok=True):
+def makedirs(name , mode  = 0o755, exist_ok  = True)  :
     bname = fsenc(name)
     try:
         os.makedirs(bname, mode)
@@ -31,32 +34,43 @@ def makedirs(name, mode=0o755, exist_ok=True):
             raise
 
 
-def mkdir(p, mode=0o755):
+def mkdir(p , mode  = 0o755)  :
     return os.mkdir(fsenc(p), mode)
 
 
-def rename(src, dst):
+def rename(src , dst )  :
     return os.rename(fsenc(src), fsenc(dst))
 
 
-def replace(src, dst):
+def replace(src , dst )  :
     return os.replace(fsenc(src), fsenc(dst))
 
 
-def rmdir(p):
+def rmdir(p )  :
     return os.rmdir(fsenc(p))
 
 
-def stat(p):
+def stat(p )  :
     return os.stat(fsenc(p))
 
 
-def unlink(p):
+def unlink(p )  :
     return os.unlink(fsenc(p))
 
 
-def utime(p, times=None, follow_symlinks=True):
+def utime(
+    p , times   = None, follow_symlinks  = True
+)  :
     if SYMTIME:
         return os.utime(fsenc(p), times, follow_symlinks=follow_symlinks)
     else:
         return os.utime(fsenc(p), times)
+
+
+if hasattr(os, "lstat"):
+
+    def lstat(p )  :
+        return os.lstat(fsenc(p))
+
+else:
+    lstat = stat
