@@ -1,14 +1,34 @@
 # coding: utf-8
 from __future__ import print_function, unicode_literals
 
-import time
 import tempfile
 from datetime import datetime
 
 from .bos import bos
 
+try:
+    raise Exception()
 
-def errdesc(errors):
+    from .util import NamedLogger
+except:
+    pass
+
+
+class StreamArc(object):
+    def __init__(
+        self,
+        log ,
+        fgen    ,
+        **kwargs 
+    ):
+        self.log = log
+        self.fgen = fgen
+
+    def gen(self)    :
+        pass
+
+
+def errdesc(errors  )    :
     report = ["copyparty failed to add the following files to the archive:", ""]
 
     for fn, err in errors:
