@@ -12,6 +12,7 @@ import shutil
 import signal
 import stat
 import subprocess as sp
+import tempfile
 import threading
 import time
 import traceback
@@ -31,6 +32,8 @@ from .util import (
     ProgressPrinter,
     absreal,
     atomic_move,
+    db_ex_chk,
+    djoin,
     fsenc,
     min_ex,
     quotep,
@@ -69,6 +72,8 @@ class Dbw(object):
 
 
 class Mpqe(object):
+    """pending files to tag-scan"""
+
     def __init__(
         self,
         mtp  ,
@@ -102,6 +107,7 @@ class Up2k(object):
         self.pp  = None
         self.rescan_cond = threading.Condition()
         self.need_rescan  = set()
+        self.db_act = 0.0
 
         self.registry     = {}
         self.flags    = {}
@@ -127,6 +133,8 @@ class Up2k(object):
         self.mem_cur = None
         self.sqlite_ver = None
         self.no_expr_idx = False
+        self.timeout = int(max(self.args.srch_time, 5) * 1.2) + 1
+        self.spools  = set()
         if HAVE_SQLITE3:
             # mojibake detector
             self.mem_cur = self._orz(":memory:")
@@ -135,7 +143,8 @@ class Up2k(object):
             if self.sqlite_ver < (3, 9):
                 self.no_expr_idx = True
         else:
-            self.log("could not initialize sqlite3, will use in-memory registry only")
+            t = "could not initialize sqlite3, will use in-memory registry only"
+            self.log(t, 3)
 
         if ANYWIN:
             # usually fails to set lastmod too quickly
@@ -196,11 +205,13 @@ class Up2k(object):
 
     def _block(self, why )  :
         self.blocked = why
-        self.log("uploads are temporarily blocked due to " + why, 3)
+        self.log("uploads temporarily blocked due to " + why, 3)
 
     def _unblock(self)  :
-        self.blocked = None
-        self.log("uploads are now possible", 2)
+        if self.blocked is not None:
+            self.blocked = None
+            if not self.stop:
+                self.log("uploads are now possible", 2)
 
     def get_state(self)  :
         mtpq   = 0
@@ -222,6 +233,9 @@ class Up2k(object):
             "hashq": self.n_hashq,
             "tagq": self.n_tagq,
             "mtpq": mtpq,
+            "dbwt": "{:.2f}".format(
+                min(1000 * 24 * 60 * 60 - 1, time.time() - self.db_act)
+            ),
         }
         return json.dumps(ret, indent=4)
 
@@ -254,10 +268,15 @@ class Up2k(object):
                 continue
 
             if self.pp:
-                cooldown = now + 5
+                cooldown = now + 1
                 continue
 
-            timeout = now + 9001
+            if self.args.no_lifetime:
+                timeout = now + 9001
+            else:
+                # important; not deferred by db_act
+                timeout = self._check_lifetimes()
+
             with self.mutex:
                 for vp, vol in sorted(self.asrv.vfs.all_vols.items()):
                     maxage = vol.flags.get("scan")
@@ -273,6 +292,20 @@ class Up2k(object):
 
                     timeout = min(timeout, deadline)
 
+            if self.db_act > now - self.args.db_act:
+                # recent db activity; defer volume rescan
+                act_timeout = self.db_act + self.args.db_act
+                if self.need_rescan:
+                    timeout = now
+
+                if timeout < act_timeout:
+                    timeout = act_timeout
+                    t = "volume rescan deferred {:.1f} sec, due to database activity"
+                    self.log(t.format(timeout - now))
+
+                continue
+
+            with self.mutex:
                 vols = list(sorted(self.need_rescan))
                 self.need_rescan.clear()
 
@@ -288,9 +321,10 @@ class Up2k(object):
                 for v in vols:
                     volage[v] = now
 
-            if self.args.no_lifetime:
-                continue
-
+    def _check_lifetimes(self)  :
+        now = time.time()
+        timeout = now + 9001
+        if now:  # diff-golf
             for vp, vol in sorted(self.asrv.vfs.all_vols.items()):
                 lifetime = vol.flags.get("lifetime")
                 if not lifetime:
@@ -337,6 +371,8 @@ class Up2k(object):
                 if hits:
                     timeout = min(timeout, now + lifetime - (now - hits[0]))
 
+        return timeout
+
     def _vis_job_progress(self, job  )  :
         perc = 100 - (len(job["need"]) * 100.0 / len(job["hash"]))
         path = os.path.join(job["ptop"], job["prel"], job["name"])
@@ -456,10 +492,9 @@ class Up2k(object):
 
             self.volstate[vol.vpath] = t
 
-        # file contents verification
-        if next((zv for zv in vols if "e2v" in zv.flags), None):
-            self._block("integrity verification")
+        self._unblock()
 
+        # file contents verification
         for vol in vols:
             if self.stop:
                 break
@@ -483,9 +518,6 @@ class Up2k(object):
 
             self.volstate[vol.vpath] = t
 
-        if self.blocked:
-            self._unblock()
-
         # open the rest + do any e2ts(a)
         needed_mutagen = False
         for vol in vols:
@@ -641,10 +673,15 @@ class Up2k(object):
         top = vol.realpath
         rei = vol.flags.get("noidx")
         reh = vol.flags.get("nohash")
+
+        dev = 0
+        if vol.flags.get("xdev"):
+            dev = bos.stat(top).st_dev
+
         with self.mutex:
             reg = self.register_vpath(top, vol.flags)
             assert reg and self.pp
-            cur, _ = reg
+            cur, db_path = reg
 
             db = Dbw(cur, 0, time.time())
             self.pp.n = next(db.c.execute("select count(w) from up"))[0]
@@ -658,15 +695,31 @@ class Up2k(object):
             excl += list(self.asrv.vfs.histtab.values())
             if WINDOWS:
                 excl = [x.replace("/", "\\") for x in excl]
+            else:
+                # ~/.wine/dosdevices/z:/ and such
+                excl += ["/dev", "/proc", "/run", "/sys"]
 
             rtop = absreal(top)
             n_add = n_rm = 0
             try:
-                n_add = self._build_dir(db, top, set(excl), top, rtop, rei, reh, [])
-                n_rm = self._drop_lost(db.c, top)
-            except:
+                n_add = self._build_dir(
+                    db,
+                    top,
+                    set(excl),
+                    top,
+                    rtop,
+                    rei,
+                    reh,
+                    [],
+                    dev,
+                    bool(vol.flags.get("xvol")),
+                )
+                n_rm = self._drop_lost(db.c, top, excl)
+            except Exception as ex:
                 t = "failed to index volume [{}]:\n{}"
                 self.log(t.format(top, min_ex()), c=1)
+                if db_ex_chk(self.log, ex, db_path):
+                    self.hub.log_stacks()
 
             if db.n:
                 self.log("commit {} new files".format(db.n))
@@ -685,7 +738,13 @@ class Up2k(object):
         rei ,
         reh ,
         seen ,
+        dev ,
+        xvol ,
     )  :
+        if xvol and not rcdir.startswith(top):
+            self.log("skip xvol: [{}] -> [{}]".format(top, rcdir), 6)
+            return 0
+
         if rcdir in seen:
             t = "bailing from symlink loop,\n  prev: {}\n  curr: {}\n  from: {}"
             self.log(t.format(seen[-1], rcdir, cdir), 3)
@@ -695,6 +754,7 @@ class Up2k(object):
         assert self.pp and self.mem_cur
         self.pp.msg = "a{} {}".format(self.pp.n, cdir)
         ret = 0
+        unreg  = []
         seen_files = {}  # != inames; files-only for dropcheck
         g = statdir(self.log_func, not self.args.no_scandir, False, cdir)
         gl = sorted(g)
@@ -704,7 +764,12 @@ class Up2k(object):
                 return -1
 
             abspath = os.path.join(cdir, iname)
+            rp = abspath[len(top) :].lstrip("/")
+            if WINDOWS:
+                rp = rp.replace("\\", "/").strip("/")
+
             if rei and rei.search(abspath):
+                unreg.append(rp)
                 continue
 
             nohash = reh.search(abspath) if reh else False
@@ -712,14 +777,20 @@ class Up2k(object):
             sz = inf.st_size
             if stat.S_ISDIR(inf.st_mode):
                 rap = absreal(abspath)
+                if dev and inf.st_dev != dev:
+                    self.log("skip xdev {}->{}: {}".format(dev, inf.st_dev, abspath), 6)
+                    continue
                 if abspath in excl or rap in excl:
+                    unreg.append(rp)
                     continue
                 if iname == ".th" and bos.path.isdir(os.path.join(abspath, "top")):
                     # abandoned or foreign, skip
                     continue
                 # self.log(" dir: {}".format(abspath))
                 try:
-                    ret += self._build_dir(db, top, excl, abspath, rap, rei, reh, seen)
+                    ret += self._build_dir(
+                        db, top, excl, abspath, rap, rei, reh, seen, dev, xvol
+                    )
                 except:
                     t = "failed to index subdir [{}]:\n{}"
                     self.log(t.format(abspath, min_ex()), c=1)
@@ -728,10 +799,6 @@ class Up2k(object):
             else:
                 # self.log("file: {}".format(abspath))
                 seen_files[iname] = 1
-                rp = abspath[len(top) :].lstrip("/")
-                if WINDOWS:
-                    rp = rp.replace("\\", "/").strip("/")
-
                 if rp.endswith(".PARTIAL") and time.time() - lmod < 60:
                     # rescan during upload
                     continue
@@ -788,6 +855,9 @@ class Up2k(object):
                         self.log("hash: {} @ [{}]".format(repr(ex), abspath))
                         continue
 
+                    if not hashes:
+                        return -1
+
                     wark = up2k_wark_from_hashlist(self.salt, sz, hashes)
 
                 self.db_add(db.c, wark, rd, fn, lmod, sz, "", 0)
@@ -803,6 +873,25 @@ class Up2k(object):
         if self.stop:
             return -1
 
+        # drop shadowed folders
+        for rd in unreg:
+            n = 0
+            q = "select count(w) from up where (rd = ? or rd like ?||'%') and at == 0"
+            for erd in [rd, "//" + w8b64enc(rd)]:
+                try:
+                    n = db.c.execute(q, (erd, erd + "/")).fetchone()[0]
+                    break
+                except:
+                    pass
+
+            if n:
+                t = "forgetting {} shadowed autoindexed files in [{}] > [{}]"
+                self.log(t.format(n, top, rd))
+
+                q = "delete from up where (rd = ? or rd like ?||'%') and at == 0"
+                db.c.execute(q, (erd, erd + "/"))
+                ret += n
+
         # drop missing files
         rd = cdir[len(top) + 1 :].strip("/")
         if WINDOWS:
@@ -825,12 +914,13 @@ class Up2k(object):
 
         return ret
 
-    def _drop_lost(self, cur , top )  :
+    def _drop_lost(self, cur , top , excl )  :
         rm = []
         n_rm = 0
         nchecked = 0
         assert self.pp
-        # `_build_dir` did all the files, now do dirs
+
+        # `_build_dir` did all unshadowed files; first do dirs:
         ndirs = next(cur.execute("select count(distinct rd) from up"))[0]
         c = cur.execute("select distinct rd from up order by rd desc")
         for (drd,) in c:
@@ -850,23 +940,55 @@ class Up2k(object):
 
             rm.append(drd)
 
-        if not rm:
-            return 0
+        if rm:
+            q = "select count(w) from up where rd = ?"
+            for rd in rm:
+                n_rm += next(cur.execute(q, (rd,)))[0]
 
-        q = "select count(w) from up where rd = ?"
-        for rd in rm:
-            n_rm += next(cur.execute(q, (rd,)))[0]
+            self.log("forgetting {} deleted dirs, {} files".format(len(rm), n_rm))
+            for rd in rm:
+                cur.execute("delete from up where rd = ?", (rd,))
 
-        self.log("forgetting {} deleted dirs, {} files".format(len(rm), n_rm))
-        for rd in rm:
-            cur.execute("delete from up where rd = ?", (rd,))
+        # then shadowed deleted files
+        n_rm2 = 0
+        c2 = cur.connection.cursor()
+        excl = [x[len(top) + 1 :] for x in excl if x.startswith(top + "/")]
+        q = "select rd, fn from up where (rd = ? or rd like ?||'%') order by rd"
+        for rd in excl:
+            for erd in [rd, "//" + w8b64enc(rd)]:
+                try:
+                    c = cur.execute(q, (erd, erd + "/"))
+                    break
+                except:
+                    pass
 
-        return n_rm
+            crd = "///"
+            cdc  = set()
+            for drd, dfn in c:
+                rd, fn = s3dec(drd, dfn)
+
+                if crd != rd:
+                    crd = rd
+                    try:
+                        cdc = set(os.listdir(os.path.join(top, rd)))
+                    except:
+                        cdc.clear()
+
+                if fn not in cdc:
+                    q = "delete from up where rd = ? and fn = ?"
+                    c2.execute(q, (drd, dfn))
+                    n_rm2 += 1
+
+        if n_rm2:
+            self.log("forgetting {} shadowed deleted files".format(n_rm2))
+
+        c2.close()
+        return n_rm + n_rm2
 
     def _verify_integrity(self, vol )  :
         """expensive; blocks database access until finished"""
         ptop = vol.realpath
-        assert self.pp and self.mtag
+        assert self.pp
 
         cur = self.cur[ptop]
         rei = vol.flags.get("noidx")
@@ -885,7 +1007,7 @@ class Up2k(object):
             qexa.append("up.rd != ? and not up.rd like ?||'%'")
             pexa.extend([vpath, vpath])
 
-        pex = tuple(pexa)
+        pex   = tuple(pexa)
         qex = " and ".join(qexa)
         if qex:
             qex = " where " + qex
@@ -900,11 +1022,22 @@ class Up2k(object):
                 b_left += sz  # sum() can overflow according to docs
                 n_left += 1
 
-            q = "select w, mt, sz, rd, fn from up" + qex
-            for w, mt, sz, drd, dfn in cur.execute(q, pex):
+            tf, _ = self._spool_warks(cur, "select w, rd, fn from up" + qex, pex, 0)
+
+        with gzip.GzipFile(mode="rb", fileobj=tf) as gf:
+            for zb in gf:
                 if self.stop:
                     return -1
 
+                w, drd, dfn = zb[:-1].decode("utf-8").split("\x00")
+                with self.mutex:
+                    q = "select mt, sz from up where w = ? and rd = ? and fn = ?"
+                    try:
+                        mt, sz = cur.execute(q, (w, drd, dfn)).fetchone()
+                    except:
+                        # file moved/deleted since spooling
+                        continue
+
                 n_left -= 1
                 b_left -= sz
                 if drd.startswith("//") or dfn.startswith("//"):
@@ -938,6 +1071,9 @@ class Up2k(object):
                         self.log("hash: {} @ [{}]".format(repr(ex), abspath))
                         continue
 
+                    if not hashes:
+                        return -1
+
                     w2 = up2k_wark_from_hashlist(self.salt, sz2, hashes)
 
                 if w == w2:
@@ -949,18 +1085,21 @@ class Up2k(object):
                 t = t.format(abspath, w, sz, mt, w2, sz2, mt2)
                 self.log(t, 1)
 
-            if e2vp and rewark:
-                self.hub.retcode = 1
-                os.kill(os.getpid(), signal.SIGTERM)
-                raise Exception("{} files have incorrect hashes".format(len(rewark)))
+        if e2vp and rewark:
+            self.hub.retcode = 1
+            os.kill(os.getpid(), signal.SIGTERM)
+            raise Exception("{} files have incorrect hashes".format(len(rewark)))
 
-            if not e2vu:
-                return 0
+        if not e2vu or not rewark:
+            return 0
 
+        with self.mutex:
             for rd, fn, w, sz, mt in rewark:
                 q = "update up set w = ?, sz = ?, mt = ? where rd = ? and fn = ? limit 1"
                 cur.execute(q, (w, sz, int(mt), rd, fn))
 
+            cur.connection.commit()
+
         return len(rewark)
 
     def _build_tags_index(self, vol )    :
@@ -968,17 +1107,12 @@ class Up2k(object):
         with self.mutex:
             reg = self.register_vpath(ptop, vol.flags)
 
-        assert reg and self.pp and self.mtag
-        _, db_path = reg
+        assert reg and self.pp
         entags = self.entags[ptop]
         flags = self.flags[ptop]
         cur = self.cur[ptop]
 
-        n_add = 0
         n_rm = 0
-        n_buf = 0
-        last_write = time.time()
-
         if "e2tsr" in flags:
             with self.mutex:
                 n_rm = cur.execute("select count(w) from mt").fetchone()[0]
@@ -989,92 +1123,164 @@ class Up2k(object):
         # integrity: drop tags for tracks that were deleted
         if "e2t" in flags:
             with self.mutex:
-                drops = []
+                n = 0
                 c2 = cur.connection.cursor()
                 up_q = "select w from up where substr(w,1,16) = ?"
+                rm_q = "delete from mt where w = ?"
                 for (w,) in cur.execute("select w from mt"):
                     if not c2.execute(up_q, (w,)).fetchone():
-                        drops.append(w[:16])
+                        c2.execute(rm_q, (w[:16],))
+                        n += 1
+
                 c2.close()
+                if n:
+                    t = "discarded media tags for {} deleted files"
+                    self.log(t.format(n))
+                    n_rm += n
 
-                if drops:
-                    msg = "discarding media tags for {} deleted files"
-                    self.log(msg.format(len(drops)))
-                    n_rm += len(drops)
-                    for w in drops:
-                        cur.execute("delete from mt where w = ?", (w,))
+        with self.mutex:
+            cur.connection.commit()
 
-        # bail if a volume flag disables indexing
+        # bail if a volflag disables indexing
         if "d2t" in flags or "d2d" in flags:
-            return n_add, n_rm, True
+            return 0, n_rm, True
 
         # add tags for new files
-        gcur = cur
-        with self.mutex:
-            gcur.connection.commit()
-
         if "e2ts" in flags:
             if not self.mtag:
-                return n_add, n_rm, False
+                return 0, n_rm, False
 
-            mpool  = None
+            nq = 0
+            with self.mutex:
+                tf, nq = self._spool_warks(
+                    cur, "select w from up order by rd, fn", (), 1
+                )
 
-            if self.mtag.prefer_mt and self.args.mtag_mt > 1:
-                mpool = self._start_mpool()
+            if not nq:
+                # self.log("tags ok")
+                self._unspool(tf)
+                return 0, n_rm, True
 
-            # TODO blocks writes to registry cursor; do chunks instead
-            conn = sqlite3.connect(db_path, timeout=15)
-            cur = conn.cursor()
-            c2 = conn.cursor()
-            c3 = conn.cursor()
-            n_left = cur.execute("select count(w) from up").fetchone()[0]
-            for w, rd, fn in cur.execute("select w, rd, fn from up order by rd, fn"):
-                if self.stop:
-                    return -1, -1, False
+            if nq == -1:
+                return -1, -1, True
 
-                n_left -= 1
-                q = "select w from mt where w = ?"
-                if c2.execute(q, (w[:16],)).fetchone():
+            with gzip.GzipFile(mode="rb", fileobj=tf) as gf:
+                n_add = self._e2ts_q(gf, nq, cur, ptop, entags)
+
+            self._unspool(tf)
+
+        return n_add, n_rm, True
+
+    def _e2ts_q(
+        self,
+        qf ,
+        nq ,
+        cur ,
+        ptop ,
+        entags ,
+    )  :
+        assert self.pp and self.mtag
+
+        flags = self.flags[ptop]
+        mpool  = None
+        if self.mtag.prefer_mt and self.args.mtag_mt > 1:
+            mpool = self._start_mpool()
+
+        n_add = 0
+        n_buf = 0
+        last_write = time.time()
+        for bw in qf:
+            if self.stop:
+                return -1
+
+            w = bw[:-1].decode("ascii")
+
+            with self.mutex:
+                try:
+                    q = "select rd, fn from up where substr(w,1,16)=? and +w=?"
+                    rd, fn = cur.execute(q, (w[:16], w)).fetchone()
+                except:
+                    # file modified/deleted since spooling
                     continue
 
-                if "mtp" in flags:
-                    q = "insert into mt values (?,'t:mtp','a')"
-                    c2.execute(q, (w[:16],))
-
                 if rd.startswith("//") or fn.startswith("//"):
                     rd, fn = s3dec(rd, fn)
 
-                abspath = os.path.join(ptop, rd, fn)
-                self.pp.msg = "c{} {}".format(n_left, abspath)
-                if not mpool:
-                    n_tags = self._tag_file(c3, entags, w, abspath)
-                else:
-                    mpool.put(Mpqe({}, entags, w, abspath, {}))
-                    # not registry cursor; do not self.mutex:
-                    n_tags = len(self._flush_mpool(c3))
+                if "mtp" in flags:
+                    q = "insert into mt values (?,'t:mtp','a')"
+                    cur.execute(q, (w[:16],))
 
-                n_add += n_tags
-                n_buf += n_tags
-
-                td = time.time() - last_write
-                if n_buf >= 4096 or td >= 60:
-                    self.log("commit {} new tags".format(n_buf))
-                    cur.connection.commit()
-                    last_write = time.time()
-                    n_buf = 0
-
-            if mpool:
-                self._stop_mpool(mpool)
+            abspath = os.path.join(ptop, rd, fn)
+            self.pp.msg = "c{} {}".format(nq, abspath)
+            if not mpool:
+                n_tags = self._tagscan_file(cur, entags, w, abspath)
+            else:
+                mpool.put(Mpqe({}, entags, w, abspath, {}))
                 with self.mutex:
-                    n_add += len(self._flush_mpool(c3))
+                    n_tags = len(self._flush_mpool(cur))
 
-            conn.commit()
-            c3.close()
-            c2.close()
-            cur.close()
-            conn.close()
+            n_add += n_tags
+            n_buf += n_tags
+            nq -= 1
 
-        return n_add, n_rm, True
+            td = time.time() - last_write
+            if n_buf >= 4096 or td >= max(1, self.timeout - 1):
+                self.log("commit {} new tags".format(n_buf))
+                with self.mutex:
+                    cur.connection.commit()
+
+                last_write = time.time()
+                n_buf = 0
+
+        if mpool:
+            self._stop_mpool(mpool)
+            with self.mutex:
+                n_add += len(self._flush_mpool(cur))
+
+        with self.mutex:
+            cur.connection.commit()
+
+        return n_add
+
+    def _spool_warks(
+        self,
+        cur ,
+        q ,
+        params  ,
+        flt ,
+    )   :
+        """mutex me"""
+        n = 0
+        c2 = cur.connection.cursor()
+        tf = tempfile.SpooledTemporaryFile(1024 * 1024 * 8, "w+b", prefix="cpp-tq-")
+        with gzip.GzipFile(mode="wb", fileobj=tf) as gf:
+            for row in cur.execute(q, params):
+                if self.stop:
+                    return tf, -1
+
+                if flt == 1:
+                    q = "select w from mt where w = ?"
+                    if c2.execute(q, (row[0][:16],)).fetchone():
+                        continue
+
+                gf.write("{}\n".format("\x00".join(row)).encode("utf-8"))
+                n += 1
+
+        c2.close()
+        tf.seek(0)
+        self.spools.add(tf)
+        return tf, n
+
+    def _unspool(self, tf )  :
+        try:
+            self.spools.remove(tf)
+        except:
+            return
+
+        try:
+            tf.close()
+        except Exception as ex:
+            self.log("failed to delete spool: {}".format(ex), 3)
 
     def _flush_mpool(self, wcur )  :
         ret = []
@@ -1335,21 +1541,38 @@ class Up2k(object):
         msg = "{} failed to read tags from {}:\n{}".format(parser, abspath, ex)
         self.log(msg.lstrip(), c=1 if "<Signals.SIG" in msg else 3)
 
+    def _tagscan_file(
+        self,
+        write_cur ,
+        entags ,
+        wark ,
+        abspath ,
+    )  :
+        """will mutex"""
+        assert self.mtag
+
+        if not bos.path.isfile(abspath):
+            return 0
+
+        try:
+            tags = self.mtag.get(abspath)
+        except Exception as ex:
+            self._log_tag_err("", abspath, ex)
+            return 0
+
+        with self.mutex:
+            return self._tag_file(write_cur, entags, wark, abspath, tags)
+
     def _tag_file(
         self,
         write_cur ,
         entags ,
         wark ,
         abspath ,
-        tags    = None,
+        tags   ,
     )  :
+        """mutex me"""
         assert self.mtag
-        if tags is None:
-            try:
-                tags = self.mtag.get(abspath)
-            except Exception as ex:
-                self._log_tag_err("", abspath, ex)
-                return 0
 
         if not bos.path.isfile(abspath):
             return 0
@@ -1379,8 +1602,7 @@ class Up2k(object):
         return ret
 
     def _orz(self, db_path )  :
-        timeout = int(max(self.args.srch_time, 5) * 1.2)
-        return sqlite3.connect(db_path, timeout, check_same_thread=False).cursor()
+        return sqlite3.connect(db_path, self.timeout, check_same_thread=False).cursor()
         # x.set_trace_callback(trace)
 
     def _open_db(self, db_path )  :
@@ -1523,11 +1745,10 @@ class Up2k(object):
                 self._job_volchk(cj)
 
         cj["name"] = sanitize_fn(cj["name"], "", [".prologue.html", ".epilogue.html"])
-        cj["poke"] = time.time()
+        cj["poke"] = now = self.db_act = time.time()
         wark = self._get_wark(cj)
-        now = time.time()
         job = None
-        pdir = os.path.join(cj["ptop"], cj["prel"])
+        pdir = djoin(cj["ptop"], cj["prel"])
         try:
             dev = bos.stat(pdir).st_dev
         except:
@@ -1639,7 +1860,7 @@ class Up2k(object):
                         for k in ["ptop", "vtop", "prel"]:
                             job[k] = cj[k]
 
-                        pdir = os.path.join(cj["ptop"], cj["prel"])
+                        pdir = djoin(cj["ptop"], cj["prel"])
                         job["name"] = self._untaken(pdir, cj["name"], now, cj["addr"])
                         dst = os.path.join(job["ptop"], job["prel"], job["name"])
                         if not self.args.nw:
@@ -1655,9 +1876,9 @@ class Up2k(object):
             if not job:
                 vfs = self.asrv.vfs.all_vols[cj["vtop"]]
                 if vfs.lim:
-                    ap1 = os.path.join(cj["ptop"], cj["prel"])
+                    ap1 = djoin(cj["ptop"], cj["prel"])
                     ap2, cj["prel"] = vfs.lim.all(
-                        cj["addr"], cj["prel"], cj["size"], ap1
+                        cj["addr"], cj["prel"], cj["size"], ap1, reg
                     )
                     bos.makedirs(ap2)
                     vfs.lim.nup(cj["addr"])
@@ -1693,7 +1914,11 @@ class Up2k(object):
                         job["need"].append(k)
                         lut[k] = 1
 
-                self._new_upload(job)
+                try:
+                    self._new_upload(job)
+                except:
+                    self.registry[job["ptop"]].pop(job["wark"], None)
+                    raise
 
             purl = "{}/{}".format(job["vtop"], job["prel"]).strip("/")
             purl = "/{}/".format(purl) if purl else "/"
@@ -1785,6 +2010,7 @@ class Up2k(object):
         self, ptop , wark , chash 
     )      :
         with self.mutex:
+            self.db_act = time.time()
             job = self.registry[ptop].get(wark)
             if not job:
                 known = " ".join([x for x in self.registry[ptop].keys()])
@@ -1835,6 +2061,7 @@ class Up2k(object):
 
     def confirm_chunk(self, ptop , wark , chash )   :
         with self.mutex:
+            self.db_act = time.time()
             try:
                 job = self.registry[ptop][wark]
                 pdir = os.path.join(job["ptop"], job["prel"])
@@ -1869,6 +2096,7 @@ class Up2k(object):
             self._finish_upload(ptop, wark)
 
     def _finish_upload(self, ptop , wark )  :
+        self.db_act = time.time()
         try:
             job = self.registry[ptop][wark]
             pdir = os.path.join(job["ptop"], job["prel"])
@@ -1945,9 +2173,15 @@ class Up2k(object):
         if not cur:
             return False
 
-        self.db_rm(cur, rd, fn)
-        self.db_add(cur, wark, rd, fn, lmod, sz, ip, at)
-        cur.connection.commit()
+        try:
+            self.db_rm(cur, rd, fn)
+            self.db_add(cur, wark, rd, fn, lmod, sz, ip, at)
+            cur.connection.commit()
+        except Exception as ex:
+            x = self.register_vpath(ptop, {})
+            assert x
+            db_ex_chk(self.log, ex, x[1])
+            raise
 
         if "e2t" in self.flags[ptop]:
             self.tagq.put((ptop, wark, rd, fn))
@@ -2005,6 +2239,7 @@ class Up2k(object):
     def _handle_rm(
         self, uname , ip , vpath 
     )    :
+        self.db_act = time.time()
         try:
             permsets = [[True, False, False, True]]
             vn, rem = self.asrv.vfs.get(vpath, uname, *permsets[0])
@@ -2089,6 +2324,7 @@ class Up2k(object):
         return n_files, ok + ok2, ng + ng2
 
     def handle_mv(self, uname , svp , dvp )  :
+        self.db_act = time.time()
         svn, srem = self.asrv.vfs.get(svp, uname, True, False, True)
         svn, srem = svn.get_dbv(srem)
         sabs = svn.canonical(srem, False)
@@ -2391,6 +2627,9 @@ class Up2k(object):
         ret = []
         with open(fsenc(path), "rb", 512 * 1024) as f:
             while fsz > 0:
+                if self.stop:
+                    return []
+
                 if self.pp:
                     mb = int(fsz / 1024 / 1024)
                     self.pp.msg = "{}{} MB, {}".format(prefix, mb, path)
@@ -2413,7 +2652,7 @@ class Up2k(object):
         return ret
 
     def _new_upload(self, job  )  :
-        pdir = os.path.join(job["ptop"], job["prel"])
+        pdir = djoin(job["ptop"], job["prel"])
         if not job["size"] and bos.path.isfile(os.path.join(pdir, job["name"])):
             return
 
@@ -2635,6 +2874,9 @@ class Up2k(object):
             self.log("hashing " + abspath)
             inf = bos.stat(abspath)
             hashes = self._hashlist_from_file(abspath)
+            if not hashes:
+                return
+
             wark = up2k_wark_from_hashlist(self.salt, inf.st_size, hashes)
             with self.mutex:
                 self.idx_wark(ptop, wark, rd, fn, inf.st_mtime, inf.st_size, ip, at)
@@ -2650,6 +2892,10 @@ class Up2k(object):
 
     def shutdown(self)  :
         self.stop = True
+
+        for x in list(self.spools):
+            self._unspool(x)
+
         self.log("writing snapshot")
         self.do_snapshot()
 
